(* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: gontmodule.mli,v 1.4 2002/02/18 11:33:30 malekith Exp $
 *
 * Module interface: Gontmodule
 * Description: Manage simple Gont module system.
 *)

open Gonttree

(* Save given list of symbols and types as named module interface to given
 * file. *)
val save_module : out_channel -> (* module *) string -> symbol list ->
                  typedef list -> unit

(* Add given file/directory to module implementation or interface search
 * path. *)
val add_to_impl_search_path : string -> unit
val add_to_iface_search_path : string -> unit

(* Reset module subsystem state. *)
val reset : unit -> unit

(* Return list of global symbols and types in given module. Used by
 * Gontcodegen to check whatever everything was implemented. *)
val globals : string -> symbol list * typedef list

(* Following functions are only used by Gontbind module. After binding
 * pass, we have symbols assigned in Ref, CrossModuleRef, NamedType and 
 * similar nodes, and we have no reason to use it. *)
 
(* Look for named symbol in named module. *)
val import_symbol : symbol_kind -> 
                    (* module *) string -> (* name *) string -> 
                    symbol

(* Look for named type in named module. *)
val import_type_symbol : (* module *) string -> (* name *) string -> typedef

(* Check whatever we can read module interface file, raise Error if not. *)
val open_module : string -> unit

(* Make named archive file from list of input files. It is conceptually
 * separate operation, i.e. it can be called without reset () and similar
 * things. *)
val make_archive : string -> string list -> unit
;;
