/* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: alloc.c,v 1.2 2002/02/16 12:09:12 malekith Exp $
 *
 * Runtime environment for gont.
 *
 * Assumes compiler that allows $ in identifiers, such as GCC.
 * (It is required to have Ksi, so this ain't so important)
 */

#include <gont/runtime.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef GONT_USE_GC
#include <gc/gc.h>
#endif

static void oops(int s) __attribute__ ((__noreturn__));

static void oops(int s)
{
	fprintf(stderr, "Run out of memory, trying to "
			"allocate %d bytes.\n", (int)s);
	abort();
}

void *gont_xmalloc(ptroff s)
{
#ifdef GONT_USE_GC
	void *p = GC_malloc(s);
#else
	void *p = malloc(s);
#endif

	if (p == NULL) 
		oops(s);
	else
		return p;
}

void *gont_xmalloc0(ptroff s)
{
#ifdef GONT_USE_GC
	void *p = GC_malloc(s);
#else
	void *p = malloc(s);

	if (p)
		memset(p, '\0', s);
#endif

	if (p == NULL) 
		oops(s);
	else
		return p;
}

void *gont_xmalloc_atomic(ptroff s)
{
#ifdef GONT_USE_GC
	void *p = GC_malloc_atomic(s);
#else
	void *p = malloc(s);
#endif

	if (p == NULL) 
		oops(s);
	else
		return p;
}

void *gont_xmalloc0_atomic(ptroff s)
{
#ifdef GONT_USE_GC
	void *p = GC_malloc_atomic(s);
#else
	void *p = malloc(s);

	if (p)
		memset(p, '\0', s);
#endif

	if (p == NULL) 
		oops(s);
	else
		return p;
}
