// diff test
// $Id: diff.g,v 1.3 2002/02/15 15:07:22 malekith Exp $

/* Takes multiplication function, and one element. Returns power function,
 * with Time = O(log n). */
*('a, int) -> 'a pow(*('a, 'a) -> 'a mul, 'a one)
{
	'a f('a val, int n) {
		if (n == 0)
			return one;
			
		if (n == 1)
			return val;
		
		'a res = f(val, n / 2);

		res = mul(res, res);
		
		if (n % 2 == 1)
			res = mul(res, val);

		return res;
	}

	return f;
}

*(int,int) -> int int_pow;

union expr {
	int Const;
        void Var;
        *[expr, expr] Add;
        *[expr, expr] Sub;
        *[expr, expr] Mul;
        *[expr, expr] Div;
        *[expr, int] Pow;
}

int compute(expr e, int val) {
        switch (e) {
        case Const[x]:  return x;
        case Var[]:     return val;
        case Add[e1, e2]: return compute(e1, val) + compute(e2, val);
        case Sub[e1, e2]: return compute(e1, val) - compute(e2, val);
        case Mul[e1, e2]: return compute(e1, val) * compute(e2, val);
        case Div[e1, e2]: return compute(e1, val) / compute(e2, val);     
        case Pow[e1, n] : 
             int ex;
             ex = compute(e1, val);
             switch(n) {
             case 0: return 1;
             case 1: return ex;
             case x: 
                     if (n < 0) return 0;
                     else return ex * compute(Pow[e1, n-1], val);
             }

        }
}

expr diff(expr e) {
        switch e {
        case Const[_]: return Const[0];
        case Var[]:    return Const[1];
        case Add[e1, e2]: return Add[diff(e1), diff(e2)];
        case Sub[e1, e2]: return Sub[diff(e1), diff(e2)];
        case Mul[e1, e2]: 
               return Add[Mul[diff(e1), e2], Mul[e1, diff(e2)]];
        case Div[e1, e2]:
               expr up = Sub[Mul[diff(e1), e2], Mul[e1, diff(e2)]];
               expr down = Mul[e2, e2]; 
               return Div[up, down];
        case Pow[ex, n]:
               return Mul[Const[n], Mul[Pow[ex,n-1], diff(ex)]];
        }
}

expr simplify(expr e) {
	expr ex;
 	switch e {
        case Add[e1, e2]: ex = Add[simplify(e1), simplify(e2)];
        case Sub[e1, e2]: ex = Sub[simplify(e1), simplify(e2)];
        case Mul[e1, e2]: ex = Mul[simplify(e1), simplify(e2)];
        case Div[e1, e2]: ex = Div[simplify(e1), simplify(e2)];
        case Pow[exx, n]: ex =  Pow[simplify(exx),n];
        case x: ex = x;
        } 

	switch ex {
	case Mul[Const[0], _]: return Const[0];
	case Mul[_, Const[0]]: return Const[0];
	case Mul[Const[1], x]: return x;
	case Mul[x, Const[1]]: return x;
	case Add[x, Const[0]]: return x;
	case Add[Const[0], x]: return x;
	case Sub[x, Const[0]]: return x;
	case Sub[Const[0], x]: return x;
	case Mul[Const[k], Mul[Const[l], x]]:
			return Mul[Const[k * l], x];
        case Add[Const[a], Const[b]]: return Const[a+b];
        case Sub[Const[a], Const[b]]: return Const[a-b];
        case Mul[Const[a], Const[b]]: return Const[a*b];
        case Div[Const[a], Const[b]]: return Const[a/b];
        case Pow[x, 1]: return x;
        case Pow[x, 0]: return Const[1];
        case Pow[Const[x], n]: return Const[int_pow(x,n)];
	case x: return x;
	}
}

void print_exp(expr e) {
	switch e {
        case Const[x]: print_int(x);
	case Var[]:    print_string("x");
        case Mul[e1,e2]: print_string("(");
			 print_exp(e1); 
                         print_string("*");
			 print_exp(e2);
                         print_string(")");   
        case Div[e1,e2]: print_string("(");
			 print_exp(e1); 
                         print_string("/");
			 print_exp(e2);
                         print_string(")");   
        case Add[e1,e2]: print_string("(");
			 print_exp(e1); 
                         print_string("+");
			 print_exp(e2);
                         print_string(")");   
        case Sub[e1,e2]: print_string("(");
			 print_exp(e1); 
                         print_string("-");
			 print_exp(e2);
                         print_string(")");
	case Pow[ex, n]: print_exp(ex);
                         print_string("^");
  			 print_int(n);   
        }


}

int main() export_as "main"
{
	*('a) -> 'a combine(*('a) -> 'a f, *('a) -> 'a g)
	{
		return fun ('a x) -> 'a is (f(g(x)));
	}

        
        /* f_power takes function f and int n, gives function f^n */ 

	*(*('a) -> 'a, int) -> *('a) -> 'a f_power;

	f_power = pow(combine, fun ('a x) -> 'a is (x));
        int_pow = pow(fun (int x, int y) -> int is (x * y), 1);

        expr e = Sub[Add[Pow[Var, 6], Mul[Const[2], Pow[Var, 2]]], Const[1]]; 

        expr temp_ex;
        *(expr) -> expr nth_diff;

	for (int i = 0; i < 7; i = i + 1) {
	  	nth_diff = f_power(diff, i);
                temp_ex = nth_diff(e);
		temp_ex = simplify(temp_ex);
	        print_exp(temp_ex);
		print_string("\n");
	}

        print_int(compute(e, 2));
        print_string(", ");
        expr eprim = diff(e);
        print_int(compute(eprim, 2));
        print_string(", ");
        expr ebis = diff(eprim);
        print_int(compute(ebis, 2));
        print_string(".\n");

      	return 0;
}

/* 
 * expected-build-status: ok
 * run-test: yes
 * expected-output: ((x^6+(2*x^2))-1)
 * expected-output: ((6*x^5)+(4*x))
 * expected-output: ((30*x^4)+4)
 * expected-output: (120*x^3)
 * expected-output: (360*x^2)
 * expected-output: (720*x)
 * expected-output: 720
 * expected-output: 71, 200, 484.
 */
